Package: plotthis
Title: High-Level Plotting Built Upon 'ggplot2' and Other Plotting
        Packages
Version: 0.12.1
Authors@R: 
    person("Panwen", "Wang", , "pwwang@pwwang.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4614-8970"))
Description: Provides high-level API and a wide range of options to create stunning, publication-quality plots effortlessly.
    It is built upon 'ggplot2' and other plotting packages, and is designed to be easy to use and to work seamlessly with 'ggplot2' objects.
    It is particularly useful for creating complex plots with multiple layers, facets, and annotations.
    It also provides a set of functions to create plots for specific types of data, such as Venn diagrams, alluvial diagrams, and phylogenetic trees.
    The package is designed to be flexible and customizable, and to work well with the 'ggplot2' ecosystem.
    The API can be found at <https://pwwang.github.io/plotthis/reference/index.html>.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/pwwang/plotthis
        https://pwwang.github.io/plotthis/
BugReports: https://github.com/pwwang/plotthis/issues
RoxygenNote: 7.3.3
Depends: R (>= 4.2.0)
Imports: circlize, ggplot2, rlang, dplyr, tidyr, glue, forcats, gtable,
        reshape2, stringr, scales, gridtext, methods, patchwork,
        ggrepel, ggnewscale, cowplot, zoo
Suggests: plotly, testthat, Matrix, alluvial, datasets, ComplexHeatmap,
        cluster, clustree, gglogger, ggwordcloud, ggalluvial,
        ggVennDiagram (>= 1.5.0), ggupset, ggpubr, ggbeeswarm, ggforce,
        ggraph, ggridges, ggmanh, qqplotr, hexbin, igraph, iNEXT,
        scattermore, sf, terra, concaveman, plotROC, OptimalCutpoints,
        proxyC, metR
LazyData: true
Additional_repositories: https://bioconductor.org/packages/release/bioc
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-06-09 07:01:57 UTC; pwwang
Author: Panwen Wang [aut, cre] (ORCID: <https://orcid.org/0000-0002-4614-8970>)
Maintainer: Panwen Wang <pwwang@pwwang.com>
Repository: CRAN
Date/Publication: 2026-06-10 05:20:19 UTC
Built: R 4.6.0; ; 2026-06-10 06:34:31 UTC; unix
