# Generated by roxygen2: do not edit by hand

S3method(SpatShapesPlot,SpatVector)
S3method(SpatShapesPlot,data.frame)
S3method(element_grob,element_textbox)
export(AlluvialPlot)
export(AreaPlot)
export(BarPlot)
export(BeeswarmPlot)
export(BoxPlot)
export(ChordPlot)
export(CircosPlot)
export(ClustreePlot)
export(CorPairsPlot)
export(CorPlot)
export(DensityPlot)
export(DimPlot)
export(DotPlot)
export(EnrichMap)
export(EnrichNetwork)
export(FeatureDimPlot)
export(GSEAPlot)
export(GSEASummaryPlot)
export(Heatmap)
export(Histogram)
export(JitterPlot)
export(LinePlot)
export(LollipopPlot)
export(ManhattanPlot)
export(Network)
export(PieChart)
export(QQPlot)
export(ROCCurve)
export(RadarPlot)
export(RarefactionPlot)
export(RidgePlot)
export(RingPlot)
export(SankeyPlot)
export(ScatterPlot)
export(SpatImagePlot)
export(SpatMasksPlot)
export(SpatPointsPlot)
export(SpatShapesPlot)
export(SpiderPlot)
export(SplitBarPlot)
export(TrendPlot)
export(UpsetPlot)
export(VelocityPlot)
export(VennDiagram)
export(ViolinPlot)
export(VolcanoPlot)
export(WaterfallPlot)
export(WordCloudPlot)
export(element_textbox)
export(palette_this)
export(show_palettes)
export(theme_blank)
export(theme_box)
export(theme_this)
importFrom(circlize,colorRamp2)
importFrom(cowplot,get_plot_component)
importFrom(dplyr,"%>%")
importFrom(dplyr,.data)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,group_modify)
importFrom(dplyr,if_else)
importFrom(dplyr,last)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,slice_sample)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(forcats,fct_relabel)
importFrom(ggnewscale,new_scale_color)
importFrom(ggnewscale,new_scale_fill)
importFrom(ggplot2,.pt)
importFrom(ggplot2,CoordPolar)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,annotate)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,as_labeller)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,coord_polar)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,draw_key_point)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_grob)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_curve)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_density_2d)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_hex)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_linerange)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guide_none)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer_data)
importFrom(ggplot2,layer_scales)
importFrom(ggplot2,margin)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_dodge2)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,position_jitterdodge)
importFrom(ggplot2,position_nudge)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,scale_alpha_continuous)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_discrete)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_linewidth)
importFrom(ggplot2,scale_linewidth_continuous)
importFrom(ggplot2,scale_shape_discrete)
importFrom(ggplot2,scale_shape_identity)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_x_reverse)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,stat_bin)
importFrom(ggplot2,stat_density_2d)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,stat_summary_hex)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,vars)
importFrom(ggplot2,waiver)
importFrom(ggplot2,wrap_dims)
importFrom(ggplot2,ylim)
importFrom(ggplot2,zeroGrob)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(glue,glue)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(grid,arrow)
importFrom(grid,convertUnit)
importFrom(grid,gList)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.grabExpr)
importFrom(grid,grid.lines)
importFrom(grid,grid.points)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,grobHeight)
importFrom(grid,grobTree)
importFrom(grid,grobWidth)
importFrom(grid,is.grob)
importFrom(grid,is.unit)
importFrom(grid,linesGrob)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(gridtext,textbox_grob)
importFrom(gtable,gtable_add_cols)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_add_rows)
importFrom(gtable,is.gtable)
importFrom(methods,formalArgs)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(patchwork,wrap_elements)
importFrom(patchwork,wrap_plots)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,as_name)
importFrom(rlang,dots_n)
importFrom(rlang,eval_tidy)
importFrom(rlang,parse_expr)
importFrom(rlang,parse_exprs)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(scales,alpha)
importFrom(scales,breaks_extended)
importFrom(scales,pretty_breaks)
importFrom(scales,rescale)
importFrom(scales,scientific)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_wrap)
importFrom(tidyr,complete)
importFrom(tidyr,expand_grid)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,uncount)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(utils,combn)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(zoo,na.approx)
