% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rgba_to_rgb}
\alias{rgba_to_rgb}
\title{Convert RGBA to RGB}
\usage{
rgba_to_rgb(RGBA, BackGround = c(1, 1, 1))
}
\description{
Convert RGBA to RGB
}
\keyword{internal}
