% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{blend_rgblist}
\alias{blend_rgblist}
\title{Blend a list of colors}
\usage{
blend_rgblist(Clist, mode = "blend", RGB_BackGround = c(1, 1, 1))
}
\description{
Blend a list of colors
}
\keyword{internal}
