% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{process_linkedheatmap_data}
\alias{process_linkedheatmap_data}
\title{Process data for LinkedHeatmap}
\usage{
process_linkedheatmap_data(
  data,
  split_by,
  split_by_sep,
  rows_split_by,
  rows_split_by_sep,
  rows_split_name,
  left_values_by,
  left_name,
  left_rows_by,
  left_rows_by_sep,
  left_rows_name,
  left_rows_orderby,
  left_columns_orderby,
  left_columns_by,
  left_columns_by_sep,
  left_columns_name,
  left_columns_split_by,
  left_columns_split_by_sep,
  left_columns_split_name,
  left_pie_group_by,
  left_pie_group_by_sep,
  left_pie_name,
  left_rows_data,
  left_columns_data,
  right_values_by,
  right_name,
  right_rows_by,
  right_rows_by_sep,
  right_rows_name,
  right_rows_orderby,
  right_columns_orderby,
  right_columns_by,
  right_columns_by_sep,
  right_columns_name,
  right_columns_split_by,
  right_columns_split_by_sep,
  right_columns_split_name,
  right_pie_group_by,
  right_pie_group_by_sep,
  right_pie_name,
  right_rows_data,
  right_columns_data,
  keep_na,
  keep_empty
)
}
\description{
Process data for LinkedHeatmap
}
\keyword{internal}
