% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{blend_to_color}
\alias{blend_to_color}
\title{Blend two colors}
\usage{
blend_to_color(C1, C2, mode = "blend")
}
\description{
Blend two colors
}
\keyword{internal}
